from ornator.class_decorator import *
from ornator.decorator import *

__version__ = '0.0.4'
__all__ = ['AfterClassDecorator', 'AfterDecorator', 'BeforeClassDecorator', 'BeforeDecorator', 'DualClassDecorator', 'DualDecorator', 'EmptyClassDecorator', 'EmptyDecorator']


from functools import wraps
from typing import Any, Callable, Optional

class BaseDecorator:
    """Base class for all decorators"""
    def __init__(self):
        self.__pre: Optional[Callable[..., Any]] = None
        self.__pos: Optional[Callable[..., Any]] = None

    @property
    def pre(self):
        return self.__pre

    @pre.setter
    def pre(self, value):
        if not callable(value):
            raise TypeError(f"Expected callable, got {type(value)}")
        self.__pre = value

    @property
    def pos(self):
        return self.__pos

    @pos.setter
    def pos(self, value):
        if not callable(value):
            raise TypeError(f"Expected callable, got {type(value)}")
        self.__pos = value


class BeforeDecorator(BaseDecorator):
    """Executes pre_handler before the function"""
    def before(self, **decorator_kwargs):
        def outer_decorator(function):
            @wraps(function)
            def wrapper(*args, **kwargs):
                if not self.pre:
                    raise ValueError("Missing pre function")
                pre_value = self.pre(*args, **kwargs, **decorator_kwargs)
                kwargs.pop('pre', None)
                return function(pre=pre_value, *args, **kwargs)
            return wrapper
        return outer_decorator


class AfterDecorator(BaseDecorator):
    """Executes pos_handler after the function"""
    def after(self, **decorator_kwargs):
        def outer_decorator(function):
            @wraps(function)
            def wrapper(*args, **kwargs):
                if not self.pos:
                    raise ValueError("Missing pos function")
                result = function(*args, **kwargs)
                return self.pos(result, **decorator_kwargs)
            return wrapper
        return outer_decorator


class DualDecorator(BaseDecorator):
    """Executes pre_handler before and pos_handler after the function"""
    def dual(self, **decorator_kwargs):
        def outer_decorator(function):
            @wraps(function)
            def wrapper(*args, **kwargs):
                if not (self.pre and self.pos):
                    raise ValueError("Missing pre or pos functions")
                pre_value = self.pre(*args, **kwargs, **decorator_kwargs)
                kwargs.pop('pre', None)
                result = function(pre=pre_value, *args, **kwargs)
                return self.pos(result, **decorator_kwargs)
            return wrapper
        return outer_decorator


class EmptyDecorator(BaseDecorator):
    """Allows custom function handling without pre/pos restrictions"""
    def empty(self, handler: Callable[..., Any] = None, **decorator_kwargs):
        if handler and not callable(handler):
            raise TypeError(f"Expected callable handler, got {type(handler)}")
            
        def outer_decorator(function):
            @wraps(function)
            def wrapper(*args, **kwargs):
                if handler:
                    return handler(function, *args, **kwargs, **decorator_kwargs)
                return function(*args, **kwargs)
            return wrapper
        return outer_decorator



from typing import TypeVar
from functools import wraps
from typing import Type, Callable

T = TypeVar('T', bound=object)

class BaseClassDecorator:
    """Base class for all class decorators"""
    def __init__(self):
        self.__pre: Callable|None = None
        self.__pos: Callable|None = None

    @property
    def pre(self):
        return self.__pre

    @pre.setter
    def pre(self, value):
        if not callable(value):
            raise TypeError(f"Expected callable, got {type(value)}")
        self.__pre = value

    @property
    def pos(self):
        return self.__pos

    @pos.setter
    def pos(self, value):
        if not callable(value):
            raise TypeError(f"Expected callable, got {type(value)}")
        self.__pos = value


class BeforeClassDecorator(BaseClassDecorator):
    """Executes pre_handler before class instantiation"""
    def before(self, **decorator_kwargs):
        def class_decorator(cls: Type[T])->Type[T]:
            original_init = cls.__init__

            @wraps(original_init)
            def wrapped_init(instance, *args, **kwargs):
                if not self.pre:
                    raise ValueError("Missing pre function")
                pre_value = self.pre(cls, *args, **kwargs, **decorator_kwargs)
                setattr(instance, '_pre_value', pre_value)
                original_init(instance, *args, **kwargs)

            cls.__init__ = wrapped_init
            return cls
        return class_decorator


class AfterClassDecorator(BaseClassDecorator):
    """Modifies class after its definition"""
    def after(self, **decorator_kwargs):
        def class_decorator(cls: Type[T])->Type[T]:
            if not self.pos:
                raise ValueError("Missing pos function")
            
            return self.pos(cls, **decorator_kwargs)
        return class_decorator


class DualClassDecorator(BaseClassDecorator):
    """Executes pre_handler during instantiation and modifies class methods"""
    def dual(self, **decorator_kwargs):
        def class_decorator(cls: Type[T])->Type[T]:
            if not (self.pre and self.pos):
                raise ValueError("Missing pre or pos functions")

            original_init = cls.__init__

            @wraps(original_init)
            def wrapped_init(instance, *args, **kwargs):
                pre_value = self.pre(cls, *args, **kwargs, **decorator_kwargs)
                setattr(instance, '_pre_value', pre_value)
                original_init(instance, *args, **kwargs)

            cls.__init__ = wrapped_init
            self.pos(cls, **decorator_kwargs)
            return cls

        return class_decorator


class EmptyClassDecorator(BaseClassDecorator):
    """Allows custom class modification without restrictions"""
    def empty(self, handler: Callable = None, **decorator_kwargs):
        if handler and not callable(handler):
            raise TypeError(f"Expected callable handler, got {type(handler)}")

        def class_decorator(cls: Type[T])->Type[T]:
            if handler:
                return handler(cls, **decorator_kwargs)
            return cls

        return class_decorator



from setuptools import setup, find_packages

def load_requirements(filename):
    with open(filename, "r") as f:
        return f.read().splitlines()
    
#print(load_requirements("requirements.txt"))

setup(
    name='Ornator',
    version='0.0.4',
    description="""This library provides a robust set of decorators for enhancing Python code functionality. It includes:
 *Base decorator classes for extensibility *Monitoring and logging capabilities *Validation and security features *Caching and performance optimization *Support for both functions and classes""",
    long_description=open('README.md', encoding="utf-8").read(),
    long_description_content_type='text/markdown',
    author='Conectar Wali SAS',
    author_email='dev@conectarwalisas.com.co',
    url='https://github.com/ConectarWali/Ornator',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: Other/Proprietary License',
        'Operating System :: OS Independent',
    ],
    license_files=['LICENSE'],
    python_requires='>=3.9',
)
